package com.aidex.common.app.server.impl;

import com.aidex.common.app.domain.vo.LocationVO;
import com.aidex.common.app.domain.vo.SysWharfVO;
import com.aidex.common.app.server.IAppService;
import com.aidex.common.exception.SysException;
import com.aidex.common.gps.domain.FenceEntity;
import com.aidex.common.gps.domain.LocationEntity;
import com.aidex.common.gps.server.IGpsService;
import com.aidex.common.utils.StringUtils;
import com.aidex.common.utils.dist.CalcDist;
import com.aidex.framework.cache.ConfigUtils;
import com.aidex.system.domain.SysNotice;
import com.aidex.system.domain.SysWharf;
import com.aidex.system.service.SysNoticeService;
import com.aidex.system.service.SysWharfService;
import org.apache.ibatis.javassist.expr.NewArray;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class AppService implements IAppService {

    @Autowired
    private SysNoticeService sysNoticeService;
    @Autowired
    private SysWharfService sysWharfService;
    @Autowired
    private IGpsService iGpsService;


    public List<SysNotice> getCarouselList(){

        List<SysNotice> carouselList = sysNoticeService.findList(new SysNotice(){{setCarousel("1");}});
        return carouselList;
    }

    public List<SysNotice> getNoticeList(){
        List<SysNotice> noticeList = sysNoticeService.findList(new SysNotice(){{setCarousel("0");setNoticeType("1");}});
        return noticeList;
    }

    public SysNotice findCarouselAndNoticeDetails(String id){
        return sysNoticeService.get(id);
    }

    public List<SysWharf> findSysWharfList(){
        return sysWharfService.findList(new SysWharf());
    }

    /*
    * direction 方向 true正向 false反向
    * */
    public List<LocationVO> findAllLocation(Boolean direction){
        // 全部船只信息
        List<LocationEntity> locationEntities = iGpsService.getLocationBatch();
        if(locationEntities.size() <= 0)
            throw new SysException(8821,"暂无运行船只");

        // 返回结果
        List<LocationVO> locationList = new ArrayList<>();

        // 停靠速度
        Integer stopSpeed = Integer.valueOf(ConfigUtils.getConfigByKey("stop.speed").getConfigValue());

        // 正反向计算数据
        List<LocationEntity> calcList = new ArrayList<>();

        // 正反向计算条件
        String[] conditions ;

        /* 正向运行 */
        if(direction){
            // 获取正向条件
            conditions = ConfigUtils.getConfigByKey("forward.direction").getConfigValue().split("-");
            calcList = locationEntities.stream().filter(local -> ((Double.valueOf(local.getDir()) >= Double.valueOf(conditions[0])) && (Double.valueOf(local.getDir()) <= Double.valueOf(conditions[1])))).collect(Collectors.toList());
        }else {
            // 获取反向条件
            conditions = ConfigUtils.getConfigByKey("reverse.direction").getConfigValue().split("-");
            calcList = locationEntities.stream().filter(local -> ((Double.valueOf(local.getDir()) >= Double.valueOf(conditions[0])) && (Double.valueOf(local.getDir()) <= Double.valueOf(conditions[1])))).collect(Collectors.toList());
        }
        for (LocationEntity locationEntity : calcList ) {
            LocationVO locationVO = new LocationVO();
            BeanUtils.copyProperties(locationEntity,locationVO);
            locationVO.setIsArrival(Boolean.FALSE);
            // 查询全部停止船只判断是否到站
            if(Integer.valueOf(locationVO.getSpeed()) <= stopSpeed){
                locationVO.setIsArrival(Boolean.TRUE);
            }
            locationList.add(locationVO);
        }
        return locationList;
    }


    public LocationVO findAllLocationByNext(Boolean direction,String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");
        // 全部船只信息
        List<LocationEntity> locationEntities = iGpsService.getLocationBatch();
        if(locationEntities.size() <= 0)
            throw new SysException(8821,"暂无运行船只");

        // 正反向计算数据
        List<LocationVO> calcList = new ArrayList<>();
        // 停靠速度
        Integer stopSpeed = Integer.valueOf(ConfigUtils.getConfigByKey("stop.speed").getConfigValue());

        // 正反向计算条件
        String[] conditions ;

        // 默认航行平均速度
        Long defaultSpeed = 0L;
        /* 正向运行 */
        if(direction){
            // 获取正向条件
            conditions = ConfigUtils.getConfigByKey("forward.direction").getConfigValue().split("-");
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("forward.default.speed").getConfigValue());
            calcList = locationEntities.stream().filter(local -> ((Double.valueOf(local.getDir()) >= Double.valueOf(conditions[0])) && (Double.valueOf(local.getDir()) <= Double.valueOf(conditions[1])))).map(lot -> {
                LocationVO locationVO = new LocationVO();
                BeanUtils.copyProperties(lot,locationVO);
                locationVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                        Double.valueOf(lot.getLat()),Double.valueOf(lot.getLon())));
                return locationVO;
            }).sorted(Comparator.comparing(LocationVO::getDistance))
                    .collect(Collectors.toList());
        }else {
            // 获取反向条件
            conditions = ConfigUtils.getConfigByKey("reverse.direction").getConfigValue().split("-");
            defaultSpeed = Long.valueOf(ConfigUtils.getConfigByKey("reverse.default.speed").getConfigValue());
            calcList = locationEntities.stream().filter(local -> ((Double.valueOf(local.getDir()) >= Double.valueOf(conditions[0])) && (Double.valueOf(local.getDir()) <= Double.valueOf(conditions[1])))).map(lot -> {
                        LocationVO locationVO = new LocationVO();
                        BeanUtils.copyProperties(lot,locationVO);
                        locationVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                                Double.valueOf(lot.getLat()),Double.valueOf(lot.getLon())));
                        return locationVO;
                    }).sorted(Comparator.comparing(LocationVO::getDistance))
                    .collect(Collectors.toList());
        }
        LocationVO res = null;
        // 有最近的船只则进计算
        if(calcList.size() > 0) {
            // 离我最近的站点
            SysWharf wharf = this.findNearestStation(lat,lon);
            // 验证船只是否经过该站点
            for (LocationVO locationVO : calcList) {
                // 验证行驶方向--获取未到达次站点船只--避免已经过站点船只近距离出现
                if(direction){
                   Double position = CalcDist.gps2d(Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng()),Double.valueOf(locationVO.getLat()),Double.valueOf(locationVO.getLon()));
                   if(position <= 0){
                       res  = locationVO;
                       break;
                   }
                }else {
                    Double position = CalcDist.gps2d(Double.valueOf(locationVO.getLat()),Double.valueOf(locationVO.getLon()),Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng()));
                    if(position <= 0){
                        res  = locationVO;
                        break;
                    }
                }
            }
            if(res == null)
                return null;
            // 停靠时间
            Long awaitTime = 0L;
            // 默认到站停留时间
            Integer inStopTime = Integer.valueOf(ConfigUtils.getConfigByKey("pull.in.time").getConfigValue());

            // 离船最近的站点
            SysWharf wharfShip = this.findNearestStation(res.getLat(),res.getLon());
            // 验证船是否停止航行
            Boolean isStopOperation = Double.valueOf(res.getSpeed()) <= Double.valueOf(stopSpeed);
            // 为避免GPS服务与本地服务器时间不同步使用GPS服务器期间验证是否进入围栏
            Long endTime = Long.valueOf(res.getServer_time());
            // 减去默认停靠时间在停靠时间内查询进入记录
            Long startTime = endTime - (inStopTime.longValue() * 60 * 1000);
            // 查询是否进入离我最近的站点
            List<FenceEntity> fenceEntities = iGpsService.findFenceIn(res.getMacid(),wharf.getWharfNanme(),startTime,endTime);
            // 查询是否进入离船最近的站点
            List<FenceEntity> fenceShipEntities = iGpsService.findFenceIn(res.getMacid(),wharfShip.getWharfNanme(),startTime,endTime);
            res.setMyWharf(wharf);

            // 船只已经停靠入站离我最近的站
            if (isStopOperation && (fenceEntities.size() > 0 || fenceShipEntities.size() > 0)) {
                res.setStopWharf(wharfShip);
                res.setIsArrival(Boolean.TRUE);
                // 如果离我最近与停靠站点不是同一站点则计算时间
                if(!wharfShip.getWharfNanme().equals(wharf.getWharfNanme())) {
                    //如果处于停止状态则计算停留剩余时间
                    Double loadingTime = inStopTime.doubleValue() - CalcDist.calcStopTime(res.getUpdtime());
                    // 计算站点间距离
                    Double distince = CalcDist.calculateDistance(Double.valueOf(res.getLat()), Double.valueOf(res.getLon()),
                            Double.valueOf(wharf.getLat()), Double.valueOf(wharf.getLng()));
                    // 计算时间
                    Double time = Math.ceil(((distince / 100) / Double.valueOf(defaultSpeed)) * 60);
                    awaitTime = loadingTime.longValue() + time.longValue();
                }
            }else {
                // 计算站点间距离
                Double distince = CalcDist.calculateDistance(Double.valueOf(res.getLat()),Double.valueOf(res.getLon()),
                        Double.valueOf(wharf.getLat()),Double.valueOf(wharf.getLng()));
                // 船只出站后停止运行
                if(isStopOperation){
                    Double time = Math.ceil(((distince / 100) / Double.valueOf(defaultSpeed)) * 60);
                    awaitTime = time.longValue();
                }else {
                    Double time = Math.ceil(((distince / 100) / Double.valueOf(res.getSpeed())) * 60);
                    awaitTime = time.longValue();
                }
            }

            res.setAwaitTime(awaitTime);
        }

        return res;
    }

    public SysWharfVO findNearestStation(String lat, String lon){
        if(StringUtils.isEmpty(lat) || StringUtils.isEmpty(lon))
            throw new SysException(4441,"请告诉我你在哪");
        List<SysWharf> listWharf = sysWharfService.findList(new SysWharf());
        List<SysWharfVO> vos = new ArrayList<>();
        for (SysWharf sysWharf : listWharf) {
            SysWharfVO wharfVO = new SysWharfVO();
            BeanUtils.copyProperties(sysWharf,wharfVO);
            wharfVO.setDistance(CalcDist.calculateDistance(Double.valueOf(lat),Double.valueOf(lon),
                    Double.valueOf(sysWharf.getLat()),Double.valueOf(sysWharf.getLng())));
            vos.add(wharfVO);
        }
        return  vos.stream().sorted(Comparator.comparing(SysWharfVO::getDistance)).findFirst().orElse(null);
    }

}
